<?php

namespace App\Controller\OgRepository\Git;

use App\Controller\OgRepository\AbstractOgRepositoryController;
use App\Entity\GitImageRepository;
use App\Entity\Image;
use App\Entity\ImageImageRepository;
use App\Entity\ImageRepository;
use App\Model\ImageStatus;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;

class SyncAction extends AbstractOgRepositoryController
{
    /**
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ClientExceptionInterface
     */
    public function __invoke(ImageRepository $input): JsonResponse
    {
        $content = $this->createRequest('GET', 'http://' . $input->getIp() . ':8006/ogrepository/v1/git/repositories');

        if (!isset($content['repositories'])) {
            return new JsonResponse(data: 'No repositories found', status: Response::HTTP_NOT_FOUND);
        }

        $repository = $this->entityManager->getRepository(GitImageRepository::class);

        $existingRepositories = $repository->findBy(['imageRepository' => $input]);

        foreach ($content['repositories'] as $repositoryData) {
            $gitImageRepositoryEntity = $repository->findOneBy([
                'imageRepository' => $input,
                'name' => $repositoryData
            ]);

            $imageEntity = $this->entityManager->getRepository(Image::class)->findOneBy(['name' => $repositoryData]);

            if (!$imageEntity) {
                $imageEntity = new Image();
                $imageEntity->setName($repositoryData);
                $imageEntity->setRemotePc(false);
                $imageEntity->setIsGlobal(false);

                $this->entityManager->persist($imageEntity);
            }

            if (!$gitImageRepositoryEntity) {
                $gitImageRepositoryEntity = new GitImageRepository();
                $gitImageRepositoryEntity->setName($repositoryData);
                $gitImageRepositoryEntity->setStatus(ImageStatus::SUCCESS);
                $gitImageRepositoryEntity->setImageRepository($input);
                $gitImageRepositoryEntity->setImage($imageEntity);
                $gitImageRepositoryEntity->setBranch('main');
                $gitImageRepositoryEntity->setCreated(true);
                $gitImageRepositoryEntity->setCreatedAt(new \DateTime());

                $this->entityManager->persist($gitImageRepositoryEntity);
            }
        }

        $this->entityManager->flush();

        return new JsonResponse(data: $content, status: Response::HTTP_OK);
    }
}